package com.wyattjmiller.dottyrevised

import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import com.wyattjmiller.dottyrevised.DotsGrid.DotSelectionStatus
import com.wyattjmiller.dottyrevised.DotsGrid.DotsGridListener


class MainActivity : AppCompatActivity() {
    private var mGame: DotsGame? = null
    private var mDotsGrid: DotsGrid? = null
    private var mMovesRemaining: TextView? = null
    private var mScore: TextView? = null
    private var mSoundEffects: SoundEffects? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        mMovesRemaining = findViewById(R.id.movesRemaining)
        mScore = findViewById(R.id.score)
        mDotsGrid = findViewById(R.id.gameGrid)
        mDotsGrid?.setGridListener(mGridListener)
        mGame = DotsGame.instance
        mSoundEffects = SoundEffects.getInstance(applicationContext)!!
        newGame()
    }

    private val mGridListener: DotsGridListener = object : DotsGridListener {
        override fun onDotSelected(dot: Dot?, status: DotSelectionStatus?) {

            // Ignore selections when game is over
            if (mGame!!.isGameOver) return

            // Add to list of selected dots
            mGame!!.addSelectedDot(dot!!)

            if (status == DotSelectionStatus.First) {
                mSoundEffects!!.resetTones()
            }

            val addStatus = mGame!!.addSelectedDot(dot)

            if (addStatus == DotsGame.AddDotStatus.Added) {
                mSoundEffects!!.playTone(true)
            } else if (addStatus == DotsGame.AddDotStatus.Removed) {
                mSoundEffects!!.playTone(false)
            }

            // If done selecting dots then replace selected dots and display new moves and score
            if (status === DotSelectionStatus.Last) {
                if (mGame!!.selectedDots.size > 1) {
                    mDotsGrid?.animateDots()

                    // These methods must be called AFTER the animation completes
                    //mGame!!.finishMove()
                    //updateMovesAndScore()
                } else {
                    mGame!!.clearSelectedDots()
                }
            }

            // Display changes to the game
            mDotsGrid!!.invalidate()
        }

        override fun onAnimationFinished() {
            mGame!!.finishMove()
            mDotsGrid!!.invalidate()
            updateMovesAndScore()

            if (mGame!!.isGameOver) mSoundEffects!!.playGameOver()
        }
    }

    fun newGameClick(view: View?) {
        newGame()
    }

    private fun newGame() {
        mGame!!.newGame()
        mDotsGrid!!.invalidate()
        updateMovesAndScore()
    }

    private fun updateMovesAndScore() {
        mMovesRemaining!!.text = String.format(mGame!!.movesLeft.toString())
        mScore!!.text = String.format(mGame!!.score.toString())
    }
}
