package com.wyattjmiller.dottyrevised

import android.content.Context
import android.media.AudioAttributes
import android.media.AudioManager
import android.media.SoundPool
import android.os.Build
import java.util.*


class SoundEffects private constructor(private val mContext: Context) {
    private var mSoundPool: SoundPool? = null
    private val mSelectSoundIds: ArrayList<Int>
    private var mSoundIndex = 0
    private val mEndGameSoundId: Int

    fun resetTones() {
        mSoundIndex = -1
    }

    fun playTone(advance: Boolean) {
        if (advance) {
            mSoundIndex++
        } else {
            mSoundIndex--
        }
        if (mSoundIndex < 0) {
            mSoundIndex = 0
        } else if (mSoundIndex >= mSelectSoundIds.size) {
            mSoundIndex = 0
        }
        mSoundPool!!.play(mSelectSoundIds[mSoundIndex], 1f, 1f, 1, 0, 1f)
    }

    fun playGameOver() {
        mSoundPool!!.play(mEndGameSoundId, 0.5f, 0.5f, 1, 0, 1f)
    }

    companion object {
        private var mSoundEffects: SoundEffects? = null
        fun getInstance(context: Context): SoundEffects? {
            if (mSoundEffects == null) {
                mSoundEffects = SoundEffects(context)
            }
            return mSoundEffects
        }
    }

    init {
        mSoundPool = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            val attributes = AudioAttributes.Builder()
                .setUsage(AudioAttributes.USAGE_GAME)
                .setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION)
                .build()
            SoundPool.Builder()
                .setAudioAttributes(attributes)
                .build()
        } else {
            SoundPool(1, AudioManager.STREAM_MUSIC, 0)
        }

        mSelectSoundIds = ArrayList()
        mSelectSoundIds.add(mSoundPool!!.load(mContext, R.raw.note_e, 1))
        mSelectSoundIds.add(mSoundPool!!.load(mContext, R.raw.note_f, 1))
        mSelectSoundIds.add(mSoundPool!!.load(mContext, R.raw.note_f_sharp, 1))
        mSelectSoundIds.add(mSoundPool!!.load(mContext, R.raw.note_g, 1))
        mEndGameSoundId = mSoundPool!!.load(mContext, R.raw.game_over, 1)
        resetTones()
    }
}